package fr.edf.distribution.brms.remove.rule.test;

import java.util.Collection;

import org.drools.compiler.kie.builder.impl.InternalKieModule;
import org.drools.core.ClockType;
import org.drools.core.RuleBaseConfiguration;
import org.drools.core.SessionConfigurationImpl;
import org.drools.core.definitions.impl.KnowledgePackageImpl;
import org.drools.core.impl.KnowledgeBaseImpl;
import org.junit.Assert;
import org.kie.api.KieBase;
import org.kie.api.KieBaseConfiguration;
import org.kie.api.KieServices;
import org.kie.api.builder.KieBuilder;
import org.kie.api.builder.KieFileSystem;
import org.kie.api.builder.KieModule;
import org.kie.api.builder.ReleaseId;
import org.kie.api.conf.EqualityBehaviorOption;
import org.kie.api.conf.EventProcessingOption;
import org.kie.api.definition.rule.Rule;
import org.kie.api.io.Resource;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.KieSessionConfiguration;
import org.kie.api.runtime.conf.ClockTypeOption;
import org.kie.internal.builder.conf.RuleEngineOption;
import org.kie.internal.definition.KnowledgePackage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class AbstractCommonTest {

	private static final Logger LOG = LoggerFactory.getLogger(IncrementalCompilationTest.class);

	protected KieBase newKieBase(String drl) {
		KieServices ks = KieServices.Factory.get();
		ReleaseId releaseId1 = ks.newReleaseId( "org.kie", "test-upgrade", "1" );
		KieModule km = createAndDeployJar( ks, releaseId1, drl);
		KieContainer kc = ks.newKieContainer( km.getReleaseId() );
		return kc.newKieBase(newKnowledgeBaseConfiguration());
	}

	protected KieSession newSession(KieBase kieBase) {
		KieSessionConfiguration conf = new SessionConfigurationImpl();
		conf.setOption(ClockTypeOption.get(ClockType.PSEUDO_CLOCK.getId()));
		return kieBase.newKieSession(conf, null);
	}

	protected KieModule createAndDeployJar(KieServices ks, ReleaseId releaseId, String... drls) {
		byte[] jar = createJar(ks, releaseId, drls);
		// Deploy jar into the repository
		KieModule km = deployJarIntoRepository(ks, jar);
		return km;
	}


	private KieModule deployJarIntoRepository(KieServices ks, byte[] jar) {
		Resource jarRes = ks.getResources().newByteArrayResource(jar);
		KieModule km = ks.getRepository().addKieModule(jarRes);
		return km;
	}

	private byte[] createJar(KieServices ks, ReleaseId releaseId, String... drls) {
		KieFileSystem kfs = ks.newKieFileSystem().generateAndWritePomXML(releaseId);
		for (int i = 0; i < drls.length; i++) {
			if (drls[i] != null) {
				kfs.write("src/main/resources/r" + i + ".drl", drls[i]);
			}
		}
		KieBuilder kb = ks.newKieBuilder(kfs).buildAll();
		Assert.assertFalse(kb.getResults().getMessages(org.kie.api.builder.Message.Level.ERROR).toString(),
				kb.getResults().hasMessages(org.kie.api.builder.Message.Level.ERROR));
		InternalKieModule kieModule = (InternalKieModule) ks.getRepository()
				.getKieModule(releaseId);
		byte[] jar = kieModule.getBytes();
		return jar;
	}

	public KieBaseConfiguration newKnowledgeBaseConfiguration() {
		KieBaseConfiguration configuration = new RuleBaseConfiguration();
		configuration.setOption(EqualityBehaviorOption.EQUALITY);
		configuration.setOption(RuleEngineOption.PHREAK);
		configuration.setOption(EventProcessingOption.STREAM);
		return configuration;
	}

	protected void removeRule(final String ruleName, KieBase kieBase) {
		Collection<KnowledgePackage> pcks = ((KnowledgeBaseImpl) kieBase).getKnowledgePackages();
		for (KnowledgePackage p : pcks) {
			Rule rule = ((KnowledgePackageImpl) p).getRule(ruleName);
			if (rule != null) {
				try {
					kieBase.removeRule(p.getName(), rule.getName());
				} catch (RuntimeException e) {
					LOG.error("Impossible de supprimer la regle {}", ruleName, e);
					throw e;
				}
				LOG.info("La regle '{}' a ete supprime du package '{}'", rule.getName(), p.getName());
				return;
			}
		}
		LOG.info("La regle '{}' n'existe pas", ruleName);
	}

}
