package fr.edf.distribution.brms.remove.rule.test;

import org.junit.Test;
import org.kie.api.KieBase;
import org.kie.api.runtime.KieSession;

import fr.edf.distribution.brms.issue.model.DummyEvent;
import fr.edf.distribution.brms.issue.model.DummyGlobalService;
import fr.edf.distribution.brms.issue.model.OtherDummyEvent;

public class IncrementalCompilationTest extends AbstractCommonTest {

	private static final String RULE_2 = "package fr.edf.distribution.brms.common.test\n" +
			"import fr.edf.distribution.brms.issue.model.DummyEvent\n" +
			"import fr.edf.distribution.brms.issue.model.OtherDummyEvent\n" +
			"import fr.edf.distribution.brms.issue.model.DummyGlobalService\n" +
			"global DummyGlobalService dummyGlobalService\n" +
			"rule \"RG_TEST_2\"\n" +
			"    when\n" +
			"        $event: OtherDummyEvent()\n" +
			"        DummyEvent()\n" +
			"        exists DummyEvent() from dummyGlobalService.getDummyEvent($event.id)\n" +
			"    then\n" +
			"        insert(new String('insert RG_TEST_2'));\n" +
			"end";

	private static final String RULE_3 = "package fr.edf.distribution.brms.common.test\n" +
			"import fr.edf.distribution.brms.issue.model.DummyEvent\n" +
			"import fr.edf.distribution.brms.issue.model.OtherDummyEvent\n" +
			"import fr.edf.distribution.brms.issue.model.DummyGlobalService\n" +
			"global DummyGlobalService dummyGlobalService\n" +
			"rule \"RG_TEST_3\"\n" +
			"    when\n" +
			"        $event: OtherDummyEvent()\n" +
			"        (or exists DummyEvent(id != $event.id)\n" +
			"            exists DummyEvent(id != 'toto'))\n" +
			"        exists DummyEvent() from dummyGlobalService.getDummyEvent($event.id)\n" +
			"    then\n" +
			"        insert(new String('insert RG_TEST_3'));\n" +
			"end";

	private static final String RULE_4 = "package fr.edf.distribution.brms.common.test\n" +
			"import fr.edf.distribution.brms.issue.model.DummyEvent\n" +
			"import fr.edf.distribution.brms.issue.model.OtherDummyEvent\n" +
			"import fr.edf.distribution.brms.issue.model.DummyGlobalService\n" +
			"global DummyGlobalService dummyGlobalService\n" +
			"rule \"RG_TEST_4\"\n" +
			"    when\n" +
			"        $e: OtherDummyEvent(id != 'toto')\n" +
			"        (or exists DummyEvent(this != $e)\n" +
			"            exists DummyEvent(id != 'toto'))\n" +
			"        exists DummyEvent() from dummyGlobalService.getDummyEvent($e.id)\n" +
			"    then\n" +
			"        insert(new String('insert RG_TEST_4'));\n" +
			"end";


	@Test
	public void testRemoveRule_1() throws Exception {
		DummyEvent dummyEvent = new DummyEvent("dummy");
		OtherDummyEvent otherDummyEvent = new OtherDummyEvent("other");

		KieBase kieBase = newKieBase(RULE_2);
		KieSession kieSession = newSession(kieBase);

		kieSession.setGlobal("dummyGlobalService", new DummyGlobalService());

		kieSession.insert(dummyEvent);
		kieSession.insert(otherDummyEvent);
		kieSession.fireAllRules();

		removeRule("RG_TEST_2", kieBase);
	}

	@Test
	public void testRemoveRule_2() throws Exception {
		DummyEvent dummyEvent = new DummyEvent("dummy");
		OtherDummyEvent otherDummyEvent = new OtherDummyEvent("dummy");

		KieBase kieBase = newKieBase(RULE_3);
		KieSession kieSession = newSession(kieBase);

		kieSession.setGlobal("dummyGlobalService", new DummyGlobalService());

		kieSession.insert(dummyEvent);
		kieSession.insert(otherDummyEvent);
		kieSession.fireAllRules();

		removeRule("RG_TEST_3", kieBase);
	}

	@Test
	public void testRemoveRule_3() throws Exception {
		OtherDummyEvent otherDummyEvent = new OtherDummyEvent("other");

		KieBase kieBase = newKieBase(RULE_4);
		KieSession kieSession = newSession(kieBase);

		kieSession.setGlobal("dummyGlobalService", new DummyGlobalService());
		kieSession.insert(otherDummyEvent);
		kieSession.fireAllRules();

		removeRule("RG_TEST_4", kieBase);
	}

}
